@@ -1,6 +1,6 @@
 /***************************************************************************
  *   Copyright (C) 2005,2006 by Jonathan Duddington                        *
- *   jsd@clara.co.uk                                                       *
+ *   jonsd@users.sourceforge.net                                           *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -17,22 +17,21 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
-// 
+#include "StdAfx.h"
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <wctype.h>
 
+#include "speak_lib.h"
 #include "speech.h"
 #include "voice.h"
 #include "phoneme.h"
 #include "synthesize.h"
 #include "translate.h"
-#include "speak_lib.h"
 
+extern int GetAmplitude(void);
 
-/* amplitudes for syllable stress levels */
-//static int stress_amps[] = {13,13, 16,16, 16,20, 20,20 };
-//static int stress_amps_r[] = {13,13, 15,15, 15,18, 18,20 };
 
 // convert from words-per-minute to internal speed factor
 static unsigned char speed_lookup[241] = {
@@ -102,10 +101,13 @@ void SetSpeed(int control)
 		// these are used in synthesis file
 		s1 = (x * voice->speedf1)/256;
 		speed_factor1 = (256 * s1)/115;      // full speed adjustment, used for pause length
-if(speed_factor1 < 18)
-	speed_factor1 = 18;
-		speed_factor2 = 120 + (137*s1)/128;  // reduced speed adjustment, used for playing recorded sounds
-//		speed_factor2 = 128 + (128*s1)/130;  // reduced speed adjustment
+if(speed_factor1 < 17)
+	speed_factor1 = 17;
+		if(wpm >= 170)
+			speed_factor2 = 110 + (151*s1)/128;  // reduced speed adjustment, used for playing recorded sounds
+		else
+			speed_factor2 = 128 + (128*s1)/130; 
+//		speed_factor2 = 120 + (137*s1)/128;  // reduced speed adjustment, used for playing recorded sounds
 	}
 
 }  //  end of SetSpeed
@@ -125,10 +127,8 @@ void SetAmplitude(int amp)
 
 
 
-#pragma GCC visibility push(default)
-
-void espeak_SetParameter(int parameter, int value, int relative)
-{//=============================================================
+void SetParameter(int parameter, int value, int relative)
+{//======================================================
 // parameter: reset-all, amp, pitch, speed, linelength, expression, capitals, number grouping
 // relative 0=absolute  1=relative
 
@@ -155,10 +155,33 @@ void espeak_SetParameter(int parameter, int value, int relative)
 
 	case espeakVOLUME:
 		embedded_value[EMBED_A] = new_value;
+		GetAmplitude();
+		break;
+
+	case espeakPITCH:
+		if(new_value > 99) new_value = 99;
+		embedded_value[EMBED_P] = new_value;
+		break;
+
+	case espeakRANGE:
+		if(new_value > 99) new_value = 99;
+		embedded_value[EMBED_R] = new_value;
+		break;
+
+	case espeakPUNCTUATION:
+		break;
+
+	case espeakCAPITALS:
+		break;
+
+	case espeakLINELENGTH:
+		option_linelength = new_value;
+		break;
+
+	default:
 		break;
 	}
 }  // end of espeak_SetParameter
-#pragma GCC visibility pop
 
 
 
@@ -237,10 +260,12 @@ void Translator::CalcLengths()
 	int  last_pitch = 0;
 	int  pitch_start;
 	int  length_mod;
+	int  len;
 	int  env2;
 	int  end_of_clause;
 	int  embedded_ix = 0;
 	int  min_drop;
+	unsigned char *pitch_env=NULL;
 
 	for(ix=1; ix<n_phoneme_list; ix++)
 	{
@@ -267,7 +292,7 @@ void Translator::CalcLengths()
 			
 		case phSTOP:
 			last_pitch = 0;
-			if(prev->type == phSTOP || prev->type == phFRICATIVE)
+			if(prev->type == phFRICATIVE)
 				p->prepause = 20;
 			else
 			if((more_syllables > 0) || (stress < 4))
@@ -275,11 +300,11 @@ void Translator::CalcLengths()
 			else
 				p->prepause = 60;
 
-			if((langopts.word_gap==3) && (p->newword))
+			if((langopts.word_gap & 0x10) && (p->newword))
 				p->prepause = 60;
 
 			if(p->synthflags & SFLAG_LENGTHEN)
-				p->prepause += 80;
+				p->prepause += langopts.long_stop;
 			break;
 
 		case phVFRICATIVE:
@@ -304,8 +329,9 @@ void Translator::CalcLengths()
 			else
 				p->length = 256;
 
-			if((langopts.word_gap==3) && (p->newword))
+			if((langopts.word_gap & 0x10) && (p->newword))
 				p->prepause = 30;
+
 			break;
 
 		case phVSTOP:
@@ -319,7 +345,7 @@ void Translator::CalcLengths()
 
 				p->prepause = 40;
 
-				if((prev->type == phPAUSE) || (prev->type == phVOWEL) || (prev->ph->mnemonic == ('/'*256+'r')))
+				if((prev->type == phPAUSE) || (prev->type == phVOWEL)) // || (prev->ph->mnemonic == ('/'*256+'r')))
 					p->prepause = 0;
 				else
 				if(p->newword==0)
@@ -333,7 +359,7 @@ void Translator::CalcLengths()
 						p->prepause = 0;
 				}
 			}
-			if((langopts.word_gap==3) && (p->newword) && (p->prepause < 20))
+			if((langopts.word_gap & 0x10) && (p->newword) && (p->prepause < 20))
 				p->prepause = 20;
 
 			break;
@@ -433,7 +459,17 @@ p->pitch1 = p->pitch2 - 20;   // post vocalic [r/]
 
 			// calc length modifier
 			if(more_syllables==0)
-				length_mod = langopts.length_mods0[next2->ph->length_mod *10+ next->ph->length_mod];
+			{
+				len = langopts.length_mods0[next2->ph->length_mod *10+ next->ph->length_mod];
+
+				if((next->newword) && (langopts.word_gap & 0x4))
+				{
+					// consider as a pause + first phoneme of the next word
+					length_mod = (len + langopts.length_mods0[next->ph->length_mod *10+ 1])/2;
+				}
+				else
+					length_mod = len;
+			}
 			else
 			{
 				length_mod = langopts.length_mods[next2->ph->length_mod *10+ next->ph->length_mod];
@@ -479,7 +515,17 @@ if(p->type != phVOWEL)
 			// set last-pitch
 			env2 = p->env;
 			if(env2 > 1) env2++;   // version for use with preceding semi-vowel
-			pitch_start = p->pitch1 + ((p->pitch2-p->pitch1)*envelope_data[env2][0])/256;
+
+			if(p->tone_ph != 0)
+			{
+				pitch_env = LookupEnvelope(phoneme_tab[p->tone_ph]->spect);
+			}
+			else
+			{
+				pitch_env = envelope_data[env2];
+			}
+
+			pitch_start = p->pitch1 + ((p->pitch2-p->pitch1)*pitch_env[0])/256;
 
 			if(pre_sonorant || pre_voiced)
 			{
@@ -512,8 +558,7 @@ if(p->type != phVOWEL)
 				
 			if(next->type == phLIQUID)
 			{
-//				if(next2->prepause == 0)    // what was this for?  caused discon. for [er]
-					next->synthflags |= SFLAG_SEQCONTINUE;
+				next->synthflags |= SFLAG_SEQCONTINUE;
 					
 				if(next2->type == phVOWEL)
 				{
