@@ -1,6 +1,6 @@
 /***************************************************************************
  *   Copyright (C) 2005,2006 by Jonathan Duddington                        *
- *   jsd@clara.co.uk                                                       *
+ *   jonsd@users.sourceforge.net                                           *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -17,6 +17,8 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+#include "StdAfx.h"
+
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -24,18 +26,24 @@
 #include <wctype.h>
 #include <string.h>
 
+
+#include "speak_lib.h"
 #include "speech.h"
 #include "voice.h"
 #include "phoneme.h"
 #include "synthesize.h"
 #include "translate.h"
+#include "wave.h"
 
-extern const char *WordToString(unsigned int word);
+const char *version_string = "1.21  01.Mar.07";
+const int version_phdata  = 0x012001;
+
+int option_device_number = -1;
 
 // copy the current phoneme table into here
 int n_phoneme_tab;
 PHONEME_TAB *phoneme_tab[N_PHONEME_TAB];
-
+unsigned char phoneme_tab_flags[N_PHONEME_TAB];   // bit 0: not inherited
 
 unsigned int *phoneme_index=NULL;
 char *spects_data=NULL;
@@ -48,7 +56,10 @@ static int phoneme_tab_number = 0;
 
 int wavefile_ix;              // a wavefile to play along with the synthesis
 int wavefile_amp;
-static int seq_len_adjust;
+int wavefile_ix2;
+int wavefile_amp2;
+
+int seq_len_adjust;
 int vowel_transition[4];
 int vowel_transition0;
 int vowel_transition1;
@@ -102,6 +113,8 @@ int LoadPhData()
 {//=============
 	int ix;
 	int n_phonemes;
+	unsigned int version;
+	int result = 1;
 	unsigned char *p;
 
 	if(ReadPhFile((char **)(&phoneme_tab_data),"phontab") != 0)
@@ -112,6 +125,12 @@ int LoadPhData()
 		return(-1);
    wavefile_data = (unsigned char *)spects_data;
 
+	version = *((unsigned int *)spects_data);
+	if(version != version_phdata)
+	{
+		result = version;
+	}
+
 	// set up phoneme tables
 	p = phoneme_tab_data;
 	n_phoneme_tables = p[0];
@@ -132,17 +151,45 @@ int LoadPhData()
 	if(phoneme_tab_number >= n_phoneme_tables)
 		phoneme_tab_number = 0;
 
-	return(0);
+	return(result);
 }  //  end of LoadPhData
 
-#ifdef PLATFORM_RISCOS
+
 void FreePhData(void)
 {//==================
 	Free(phoneme_tab_data);
 	Free(phoneme_index);
 	Free(spects_data);
 }
-#endif
+
+
+int LookupPh(const char *string)
+{//=============================
+	int  ix;
+	unsigned char c;
+	unsigned int  mnem;
+
+	// Pack up to 4 characters into a word
+	mnem = 0;
+	for(ix=0; ix<4; ix++)
+	{
+		if(string[ix]==0) break;
+		c = string[ix];
+		mnem |= (c << (ix*8));
+	}
+
+	for(ix=0; ix<n_phoneme_tab; ix++)
+	{
+		if(phoneme_tab[ix] == NULL)
+			continue;
+		if(phoneme_tab[ix]->mnemonic == mnem)
+			return(ix);
+	}
+	return(0);
+}
+
+
+
 
 static unsigned int LookupSound2(int index, unsigned int other_phcode, int control)
 {//================================================================================
@@ -162,7 +209,9 @@ static unsigned int LookupSound2(int index, unsigned int other_phcode, int contr
 				case 0:
 					// next entry is a wavefile to be played along with the synthesis
 					if(control==0)
+					{
 						wavefile_ix = value2 >> 8;
+					}
 					break;
 				case 1:
 					if(control==0)
@@ -174,7 +223,9 @@ static unsigned int LookupSound2(int index, unsigned int other_phcode, int contr
 					break;
 				case 3:
 					if(control==0)
+					{
 						wavefile_amp = value2 >> 8;
+					}
 					break;
 				case 4:
 					// formant transition data, 2 words
@@ -320,19 +371,23 @@ frameref_t *LookupSpect(PHONEME_TAB *this_ph, PHONEME_TAB *prev_ph, PHONEME_TAB
 	else
 		other_ph = next_ph;
 
-	if((ix = LookupSound(this_ph,other_ph,which,match_level,0)) == 0)
+	if((ix = LookupSound(this_ph,other_ph,which,match_level,0)) < 4)
 		return(NULL);
 	seq = (SPECT_SEQ *)(&spects_data[ix]);
 	nf = seq->n_frames;
 
+
+	if(nf >= N_SEQ_FRAMES)
+		nf = N_SEQ_FRAMES - 1;
+
 	seq_break = 0;
 	length1 = 0;
 	for(ix=0; ix<nf; ix++)
 	{
 		frames_buf[ix].frame = &seq->frame[ix];
-		frames_buf[ix].flags = seq->frame[ix].flags;
+		frames_buf[ix].frflags = seq->frame[ix].frflags;
 		frames_buf[ix].length = seq->frame[ix].length;
-		if(seq->frame[ix].flags & FRFLAG_VOWEL_CENTRE)
+		if(seq->frame[ix].frflags & FRFLAG_VOWEL_CENTRE)
 			seq_break = ix;
 	}
 	
@@ -353,7 +408,7 @@ frameref_t *LookupSpect(PHONEME_TAB *this_ph, PHONEME_TAB *prev_ph, PHONEME_TAB
 	// do we need to modify a frame for blending with a consonant?
 	if(this_ph->type == phVOWEL)
 	{
-		if(which==2)
+		if((which==2) && ((frames[nf-1].frflags & FRFLAG_BREAK) == 0))
 		{
 			// lookup formant transition for the following phoneme
 
@@ -413,6 +468,8 @@ frameref_t *LookupSpect(PHONEME_TAB *this_ph, PHONEME_TAB *prev_ph, PHONEME_TAB
 			//   less the front part of the vowel and any added suffix
 	
 			length_std = this_ph->std_length + seq_len_adjust - 45;
+			if(length_std < 10)
+				length_std = 10;
 			if(plist->synthflags & SFLAG_LENGTHEN)
 				length_std += phoneme_tab[phonLENGTHEN]->std_length;  // phoneme was followed by an extra : symbol
 
@@ -465,17 +522,22 @@ unsigned char *LookupEnvelope(int ix)
 }
 
 
-static void SetUpPhonemeTable(int number)
-{//======================================
+static void SetUpPhonemeTable(int number, int recursing)
+{//=====================================================
 	int ix;
 	int includes;
 	int ph_code;
 	PHONEME_TAB *phtab;
 
+	if(recursing==0)
+	{
+		memset(phoneme_tab_flags,0,sizeof(phoneme_tab_flags));
+	}
+
 	if((includes = phoneme_tab_list[number].includes) > 0)
 	{
 		// recursively include base phoneme tables
-		SetUpPhonemeTable(includes-1);
+		SetUpPhonemeTable(includes-1,1);
 	}
 
 	// now add the phonemes from this table
@@ -486,22 +548,23 @@ static void SetUpPhonemeTable(int number)
 		phoneme_tab[ph_code] = &phtab[ix];
 		if(ph_code > n_phoneme_tab)
 			n_phoneme_tab = ph_code;
+
+		if(recursing == 0)
+			phoneme_tab_flags[ph_code] |= 1;   // not inherited
 	}
 }  // end of SetUpPhonemeTable
 
 
 void SelectPhonemeTable(int number)
-{//===============================
+{//================================
 	n_phoneme_tab = 0;
-	SetUpPhonemeTable(number);  // recursively for included phoneme tables
+	SetUpPhonemeTable(number,0);  // recursively for included phoneme tables
 	n_phoneme_tab++;
 }  //  end of SelectPhonemeTable
 
 
-int SelectPhonemeTableName(const char *name)
-{//=========================================
-// Look up a phoneme set by name, and select it if it exists
-// Returns the phoneme table number
+int LookupPhonemeTable(const char *name)
+{//=====================================
 	int ix;
 
 	for(ix=0; ix<n_phoneme_tables; ix++)
@@ -515,7 +578,66 @@ int SelectPhonemeTableName(const char *name)
 	if(ix == n_phoneme_tables)
 		return(-1);
 
+	return(ix);
+}
+
+
+int SelectPhonemeTableName(const char *name)
+{//=========================================
+// Look up a phoneme set by name, and select it if it exists
+// Returns the phoneme table number
+	int ix;
+
+	if((ix = LookupPhonemeTable(name)) == -1)
+		return(-1);
+
 	SelectPhonemeTable(ix);
 	return(ix);
 }  //  end of DelectPhonemeTableName
 
+
+
+
+void LoadConfig(void)
+{//==================
+// Load configuration file, if one exists
+	char buf[130];
+	FILE *f;
+	int ix;
+	char c1;
+	char *p;
+	char string[120];
+
+	sprintf(buf,"%s%c%s",path_home,PATHSEP,"config");
+	if((f = fopen(buf,"r"))==NULL)
+	{
+		return;
+	}
+
+	while(fgets(buf,sizeof(buf),f)!=NULL)
+	{
+		if(memcmp(buf,"tone",4)==0)
+		{
+			ReadTonePoints(&buf[5],tone_points);
+		}
+		else
+		if(memcmp(buf,"pa_device",9)==0)
+		{
+			sscanf(&buf[7],"%d",&option_device_number);
+		}
+		else
+		if(memcmp(buf,"soundicon",9)==0)
+		{
+			ix = sscanf(&buf[10],"_%c %s",&c1,string);
+			if(ix==2)
+			{
+				soundicon_tab[n_soundicon_tab].name = c1;
+				p = Alloc(strlen(string+1));
+				strcpy(p,string);
+				soundicon_tab[n_soundicon_tab].filename = p;
+				soundicon_tab[n_soundicon_tab++].length = 0;
+			}
+		}
+	}
+}  //  end of LoadConfig
+
