@@ -1,6 +1,6 @@
 /***************************************************************************
  *   Copyright (C) 2005, 2006 by Jonathan Duddington                       *
- *   jsd@clara.co.uk                                                       *
+ *   jonsd@users.sourceforge.net                                           *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -17,6 +17,7 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+#include "StdAfx.h"
 
 // this version keeps wavemult window as a constant fraction
 // of the cycle length - but that spreads out the HF peaks too much
@@ -27,6 +28,7 @@
 #include <math.h>
 
 
+#include "speak_lib.h"
 #include "speech.h"
 #include "voice.h"
 #include "phoneme.h"
@@ -66,6 +68,7 @@ int embedded_value[N_EMBEDDED_VALUES];
 
 static int PHASE_INC_FACTOR;
 int samplerate = 0;       // this is set by Wavegeninit()
+extern int option_device_number;
 
 static wavegen_peaks_t peaks[N_PEAKS];
 static int peak_harmonic[N_PEAKS];
@@ -77,8 +80,6 @@ static int echo_tail;
 static int echo_amp = 0;
 static short echo_buf[N_ECHO_BUF];
 
-#define N_TONE_ADJUST  1250
-static unsigned char tone_adjust[N_TONE_ADJUST];   //  8Hz steps * 1250 = 10kHz
 static int harm_sqrt_n = 0;
 
 
@@ -105,7 +106,7 @@ static int mix_wavefile_ix = 0;
 static int pitch;          // pitch Hz*256
 static int pitch_ix;       // index into pitch envelope (*256)
 static int pitch_inc;      // increment to pitch_ix
-static unsigned char *pitch_env;
+static unsigned char *pitch_env=NULL;
 static int pitch_base;     // Hz*256 low, before modified by envelope
 static int pitch_range;    // Hz*256 range of envelope
 static int amp_ix;
@@ -124,7 +125,6 @@ static int hf_factor;
 
 unsigned char *out_ptr;
 unsigned char *out_end;
-static int out_channels=1;
 static unsigned char outbuf[1024];  // used when writing to file
 
 // the queue of operations passed to wavegen from sythesize
@@ -439,6 +439,7 @@ static void WavegenInitPkData(int which)
 
 static int userdata[4];
 static PaError pa_init_err=0;
+static int out_channels=1;
 
 #if USE_PORTAUDIO == 18
 static int WaveCallback(void *inputBuffer, void *outputBuffer,
@@ -458,6 +459,22 @@ static int WaveCallback(const void *inputBuffer, void *outputBuffer,
 
 	result = WavegenFill(1);
 
+#ifdef ARCH_BIG
+	{
+		// swap the order of bytes in each sound sample in the portaudio buffer
+		int c;
+		out_ptr = (unsigned char *)outputBuffer;
+		out_end = out_ptr + framesPerBuffer*2;
+		while(out_ptr < out_end)
+		{
+			c = out_ptr[0];
+			out_ptr[0] = out_ptr[1];
+			out_ptr[1] = c;
+			out_ptr += 2;
+		}
+	}
+#endif
+
 	if(out_channels == 2)
 	{
 		// sound output can only do stereo, not mono.  Duplicate each sound sample to
@@ -492,6 +509,49 @@ static int WaveCallback(const void *inputBuffer, void *outputBuffer,
 }  //  end of WaveCallBack
 
 
+#if USE_PORTAUDIO == 19
+/* This is a fixed version of Pa_OpenDefaultStream() for use if the version in portaudio V19
+   is broken */
+
+PaError Pa_OpenDefaultStream2( PaStream** stream,
+                              int inputChannelCount,
+                              int outputChannelCount,
+                              PaSampleFormat sampleFormat,
+                              double sampleRate,
+                              unsigned long framesPerBuffer,
+                              PaStreamCallback *streamCallback,
+                              void *userData )
+{
+	PaError result;
+	PaStreamParameters hostApiOutputParameters;
+
+	if(option_device_number >= 0)
+		hostApiOutputParameters.device = option_device_number;
+	else
+		hostApiOutputParameters.device = Pa_GetDefaultOutputDevice();
+
+	if( hostApiOutputParameters.device == paNoDevice )
+		return paDeviceUnavailable; 
+
+	hostApiOutputParameters.channelCount = outputChannelCount;
+	hostApiOutputParameters.sampleFormat = sampleFormat;
+	/* defaultHighOutputLatency is used below instead of
+	   defaultLowOutputLatency because it is more important for the default
+	   stream to work reliably than it is for it to work with the lowest
+	   latency.
+	*/
+	hostApiOutputParameters.suggestedLatency =
+	      Pa_GetDeviceInfo( hostApiOutputParameters.device )->defaultHighOutputLatency;
+	hostApiOutputParameters.hostApiSpecificStreamInfo = NULL;
+
+	result = Pa_OpenStream(
+		stream, NULL, &hostApiOutputParameters, sampleRate, framesPerBuffer, paNoFlag, streamCallback, userData );
+
+	return(result);
+}
+#endif
+
+
 int WavegenOpenSound()
 {//===================
 	PaError err, err2;
@@ -525,7 +585,7 @@ int WavegenOpenSound()
 			err2 = Pa_OpenDefaultStream(&pa_stream,0,2,paInt16,samplerate,512,N_WAV_BUF,WaveCallback,(void *)userdata);
 		}
 #else
-		err2 = Pa_OpenDefaultStream(&pa_stream,0,1,paInt16,(double)samplerate,512,WaveCallback,(void *)userdata);
+		err2 = Pa_OpenDefaultStream2(&pa_stream,0,1,paInt16,(double)samplerate,512,WaveCallback,(void *)userdata);
 
 		if(err2 == paInvalidChannelCount)
 		{
@@ -536,6 +596,16 @@ int WavegenOpenSound()
 #endif
 	}
 	err = Pa_StartStream(pa_stream);
+
+#if USE_PORTAUDIO == 19
+	if(err == paStreamIsNotStopped)
+	{
+		// not sure why we need this, but PA v19 seems to need it
+		err = Pa_StopStream(pa_stream);
+		err = Pa_StartStream(pa_stream);
+	}
+#endif
+
 	if(err != paNoError)
 	{
 		// exit speak if we can't open the sound device - this is OK if speak is being run for each utterance
@@ -643,33 +713,26 @@ void WavegenInit(int rate, int wavemult_fact)
 		}
 	}
 
-	// This table provides the opportunity for tone control.
-	// Adjustment of harmonic amplitudes, steps of 8Hz
-	// value of 128 means no change
-	for(ix=0; ix<N_TONE_ADJUST; ix++)
-	{
-		tone_adjust[ix] = 128;
-	}
-
 	WavegenInitPkData(1);
 	WavegenInitPkData(0);
 	pk_shape = pk_shape2;         // ph_shape2
 
 #ifdef LOG_FRAMES
-remove("log");
+remove("log-espeakedit");
 #endif
 }  // end of WavegenInit
 
 
-static int GetAmplitude(void)
-{//==========================
+int GetAmplitude(void)
+{//===================
 	int amp;
 
 	// normal, none, reduced, moderate, strong
 	static const unsigned char amp_emphasis[5] = {16, 16, 8, 24, 32};
 
 	amp = (embedded_value[EMBED_A])*60/100;
-	return(amp * amp_emphasis[embedded_value[EMBED_F]] / 16);
+	general_amplitude = amp * amp_emphasis[embedded_value[EMBED_F]] / 16;
+	return(general_amplitude);
 }
 
 
@@ -720,6 +783,7 @@ int PeaksToHarmspect(wavegen_peaks_t *peaks, int pitch, int *htab, int control)
 	int hmax;
 	int hmax_samplerate;      // highest harmonic allowed for the samplerate
 	int x;
+	int ix;
 	int h1;
 
 #ifdef SPECT_EDITOR
@@ -782,7 +846,10 @@ int PeaksToHarmspect(wavegen_peaks_t *peaks, int pitch, int *htab, int control)
 		x = htab[h] >> 15;
 		htab[h] = (x * x) >> 7;
 
-		htab[h] = (htab[h] * tone_adjust[f >> 19]) >> 13;  // index tone_adjust with Hz/8
+		if((ix = (f >> 19)) < N_TONE_ADJUST)
+		{
+			htab[h] = (htab[h] * wvoice->tone_adjust[ix]) >> 13;  // index tone_adjust with Hz/8
+		}
 	}
 
 	// adjust the amplitude of the first harmonic, affects tonal quality
@@ -1112,7 +1179,7 @@ static int Wavegen()
 static int PlaySilence(int length, int resume)
 {//===========================================
 	static int n_samples;
-	int value;
+	int value=0;
 
 	nsamples = 0;
 	samplecount = 0;
@@ -1302,7 +1369,7 @@ void SetPitch(int length, unsigned char *env, int pitch1, int pitch2)
 	int range;
 
 #ifdef LOG_FRAMES
-f_log=fopen("log","a");
+f_log=fopen("log-espeakedit","a");
 if(f_log != NULL)
 {
 fprintf(f_log,"	  %3d %3d\n",pitch1,pitch2);
@@ -1360,12 +1427,12 @@ void SetSynth(int length, int modn, frame_t *fr1, frame_t *fr2)
 	static int glottal_reduce_tab2[4] = {0x90, 0xa0, 0xb0, 0xc0};  // vowel after [?], amp * 1/256
 
 #ifdef LOG_FRAMES
-f_log=fopen("log","a");
+f_log=fopen("log-espeakedit","a");
 if(f_log != NULL)
 {
-fprintf(f_log,"%3d   %4d %4d %4d   %4d %4d %4d\n",length*1000/samplerate,
-	fr1->ffreq[1], fr1->ffreq[2], fr1->ffreq[3],
-	fr2->ffreq[1], fr2->ffreq[2], fr2->ffreq[3] );
+fprintf(f_log,"%3dmS  %4d/%3d %4d/%3d %4d/%3d %4d/%3d   %4d/%3d %4d/%3d %4d/%3d %4d/%3d\n",length*1000/samplerate,
+	fr1->ffreq[0],fr1->fheight[0],fr1->ffreq[1],fr1->fheight[1], fr1->ffreq[2],fr1->fheight[2], fr1->ffreq[3],fr1->fheight[3],
+	fr2->ffreq[0],fr2->fheight[0],fr2->ffreq[1],fr2->fheight[1], fr2->ffreq[2],fr2->fheight[2], fr2->ffreq[3],fr2->fheight[3] );
 fclose(f_log);
 f_log=NULL;
 }
@@ -1452,13 +1519,23 @@ static int Wavegen2(int length, int modulation, int resume, frame_t *fr1, frame_
 	return(Wavegen());
 }
 
+void Write4Bytes(FILE *f, int value)
+{//=================================
+// Write 4 bytes to a file, least significant first
+	int ix;
+
+	for(ix=0; ix<4; ix++)
+	{
+		fputc(value & 0xff,f);
+		value = value >> 8;
+	}
+}
+
 
 
 int OpenWaveFile(const char *path, int rate)
 /******************************************/
 {
-	int *p;
-
 	static unsigned char wave_hdr[44] = {
 		'R','I','F','F',0,0,0,0,'W','A','V','E','f','m','t',' ',
 		0x10,0,0,0,1,0,1,0,  9,0x3d,0,0,0x12,0x7a,0,0,
@@ -1470,11 +1547,6 @@ int OpenWaveFile(const char *path, int rate)
 
 	wavephase = 0x7fffffff;
 
-	// set the sample rate in the header
-	p = (int *)(&wave_hdr[24]);
-	p[0] = rate;
-	p[1] = rate * 2;
-
 	if(strcmp(path,"stdout")==0)
 		f_wave = stdout;
 	else
@@ -1482,7 +1554,10 @@ int OpenWaveFile(const char *path, int rate)
 
 	if(f_wave != NULL)
 	{
-		fwrite(wave_hdr,1,sizeof(wave_hdr),f_wave);
+		fwrite(wave_hdr,1,24,f_wave);
+		Write4Bytes(f_wave,rate);
+		Write4Bytes(f_wave,rate * 2);
+		fwrite(&wave_hdr[32],1,8,f_wave);
 		return(0);
 	}
 	return(1);
@@ -1495,7 +1570,6 @@ void CloseWaveFile(int rate)
 /******************/
 {
    unsigned int pos;
-   static int value;
 
    if(f_log != NULL)
 	{
@@ -1509,21 +1583,12 @@ void CloseWaveFile(int rate)
    fflush(f_wave);
    pos = ftell(f_wave);
 
-   value = pos - 8;
-   fseek(f_wave,4,SEEK_SET);
-   fwrite(&value,4,1,f_wave);
+	fseek(f_wave,4,SEEK_SET);
+	Write4Bytes(f_wave,pos - 8);
 
-	value = rate;
-	fseek(f_wave,24,SEEK_SET);
-	fwrite(&value,4,1,f_wave);
+	fseek(f_wave,40,SEEK_SET);
+	Write4Bytes(f_wave,pos - 44);
 
-	value = rate*2;
-	fseek(f_wave,28,SEEK_SET);
-	fwrite(&value,4,1,f_wave);
-
-   value = pos - 44;
-   fseek(f_wave,40,SEEK_SET);
-   fwrite(&value,4,1,f_wave);
 
    fclose(f_wave);
    f_wave = NULL;
